#!/bin/sh
# Test mkdir -pv.

# Copyright (C) 2006-2016 Free Software Foundation, Inc.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

. "${srcdir=.}/tests/init.sh"; path_prepend_ ./src
print_ver_ mkdir

mkdir -pv foo/a/b/c/d >out || fail=1

compare - out <<\EOF
mkdir: created directory 'foo'
mkdir: created directory 'foo/a'
mkdir: created directory 'foo/a/b'
mkdir: created directory 'foo/a/b/c'
mkdir: created directory 'foo/a/b/c/d'
EOF

Exit $fail
